@@ -6,19 +6,19 @@
  fee is hereby granted, provided that the above copyright
  notice appear in all copies and that both that copyright
  notice and this permission notice appear in supporting
- documentation, and that the name of Silicon Graphics not be 
- used in advertising or publicity pertaining to distribution 
+ documentation, and that the name of Silicon Graphics not be
+ used in advertising or publicity pertaining to distribution
  of the software without specific prior written permission.
- Silicon Graphics makes no representation about the suitability 
+ Silicon Graphics makes no representation about the suitability
  of this software for any purpose. It is provided "as is"
  without any express or implied warranty.
- 
- SILICON GRAPHICS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS 
- SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY 
+
+ SILICON GRAPHICS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
+ SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
  AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL SILICON
- GRAPHICS BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
- DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, 
- DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE 
+ GRAPHICS BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
+ DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
+ DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
  OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
  THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
@@ -339,25 +339,26 @@ MergeKeyGroups(SymbolsInfo * info,
     clobber = (from->defs.merge != MergeAugment);
     report = (warningLevel > 9) ||
         ((into->defs.fileID == from->defs.fileID) && (warningLevel > 0));
-    if (into->numLevels[group] >= from->numLevels[group])
-    {
-        resultSyms = into->syms[group];
-        resultActs = into->acts[group];
-        resultWidth = into->numLevels[group];
-    }
-    else
+    if ((from->numLevels[group] > into->numLevels[group])
+        || (clobber && (from->types[group] != None)))
     {
         resultSyms = from->syms[group];
         resultActs = from->acts[group];
         resultWidth = from->numLevels[group];
     }
+    else
+    {
+        resultSyms = into->syms[group];
+        resultActs = into->acts[group];
+        resultWidth = into->numLevels[group];
+    }
     if (resultSyms == NULL)
     {
         resultSyms = uTypedCalloc(resultWidth, KeySym);
         if (!resultSyms)
         {
             WSGO("Could not allocate symbols for group merge\n");
-            ACTION2("Group %d of key %s not merged\n", group,
+            ACTION("Group %d of key %s not merged\n", group,
                     longText(into->name, XkbMessage));
             return False;
         }
@@ -368,7 +369,7 @@ MergeKeyGroups(SymbolsInfo * info,
         if (!resultActs)
         {
             WSGO("Could not allocate actions for group merge\n");
-            ACTION2("Group %d of key %s not merged\n", group,
+            ACTION("Group %d of key %s not merged\n", group,
                     longText(into->name, XkbMessage));
             return False;
         }
@@ -403,10 +404,10 @@ MergeKeyGroups(SymbolsInfo * info,
             }
             if (report)
             {
-                WARN3
+                WARN
                     ("Multiple symbols for level %d/group %d on key %s\n",
                      i + 1, group + 1, longText(into->name, XkbMessage));
-                ACTION2("Using %s, ignoring %s\n",
+                ACTION("Using %s, ignoring %s\n",
                         XkbKeysymText(use, XkbMessage),
                         XkbKeysymText(ignore, XkbMessage));
             }
@@ -442,10 +443,10 @@ MergeKeyGroups(SymbolsInfo * info,
                 }
                 if (report)
                 {
-                    WARN3
+                    WARN
                         ("Multiple actions for level %d/group %d on key %s\n",
                          i + 1, group + 1, longText(into->name, XkbMessage));
-                    ACTION2("Using %s, ignoring %s\n",
+                    ACTION("Using %s, ignoring %s\n",
                             XkbActionTypeText(use->type, XkbMessage),
                             XkbActionTypeText(ignore->type, XkbMessage));
                 }
@@ -547,10 +548,10 @@ MergeKeys(SymbolsInfo * info, KeyInfo * into, KeyInfo * from)
                     use = into->types[i];
                     ignore = from->types[i];
                 }
-                WARN2
+                WARN
                     ("Multiple definitions for group %d type of key %s\n",
                      i, longText(into->name, XkbMessage));
-                ACTION2("Using %s, ignoring %s\n",
+                ACTION("Using %s, ignoring %s\n",
                         XkbAtomText(NULL, use, XkbMessage),
                         XkbAtomText(NULL, ignore, XkbMessage));
             }
@@ -589,9 +590,9 @@ MergeKeys(SymbolsInfo * info, KeyInfo * into, KeyInfo * from)
     }
     if (collide)
     {
-        WARN1("Symbol map for key %s redefined\n",
+        WARN("Symbol map for key %s redefined\n",
               longText(into->name, XkbMessage));
-        ACTION1("Using %s definition for conflicting fields\n",
+        ACTION("Using %s definition for conflicting fields\n",
                 (from->defs.merge == MergeAugment ? "first" : "last"));
     }
     return True;
@@ -656,10 +657,10 @@ AddModMapEntry(SymbolsInfo * info, ModMapEntry * new)
                     use = mm->modifier;
                     ignore = new->modifier;
                 }
-                ERROR1
+                ERROR
                     ("%s added to symbol map for multiple modifiers\n",
                      XkbKeysymText(new->u.keySym, XkbMessage));
-                ACTION2("Using %s, ignoring %s.\n",
+                ACTION("Using %s, ignoring %s.\n",
                         XkbModIndexText(use, XkbMessage),
                         XkbModIndexText(ignore, XkbMessage));
                 mm->modifier = use;
@@ -682,9 +683,9 @@ AddModMapEntry(SymbolsInfo * info, ModMapEntry * new)
                     use = mm->modifier;
                     ignore = new->modifier;
                 }
-                ERROR1("Key %s added to map for multiple modifiers\n",
+                ERROR("Key %s added to map for multiple modifiers\n",
                        longText(new->u.keyName, XkbMessage));
-                ACTION2("Using %s, ignoring %s.\n",
+                ACTION("Using %s, ignoring %s.\n",
                         XkbModIndexText(use, XkbMessage),
                         XkbModIndexText(ignore, XkbMessage));
                 mm->modifier = use;
@@ -696,7 +697,7 @@ AddModMapEntry(SymbolsInfo * info, ModMapEntry * new)
     if (mm == NULL)
     {
         WSGO("Could not allocate modifier map entry\n");
-        ACTION1("Modifier map for %s will be incomplete\n",
+        ACTION("Modifier map for %s will be incomplete\n",
                 XkbModIndexText(new->modifier, XkbMessage));
         return False;
     }
@@ -901,24 +902,24 @@ GetGroupIndex(KeyInfo * key,
                 return True;
             }
         }
-        ERROR3("Too many groups of %s for key %s (max %d)\n", name,
+        ERROR("Too many groups of %s for key %s (max %d)\n", name,
                longText(key->name, XkbMessage), XkbNumKbdGroups + 1);
-        ACTION1("Ignoring %s defined for extra groups\n", name);
+        ACTION("Ignoring %s defined for extra groups\n", name);
         return False;
     }
     if (!ExprResolveInteger
         (arrayNdx, &tmp, SimpleLookup, (XPointer) groupNames))
     {
-        ERROR2("Illegal group index for %s of key %s\n", name,
+        ERROR("Illegal group index for %s of key %s\n", name,
                longText(key->name, XkbMessage));
         ACTION("Definition with non-integer array index ignored\n");
         return False;
     }
     if ((tmp.uval < 1) || (tmp.uval > XkbNumKbdGroups))
     {
-        ERROR3("Group index for %s of key %s is out of range (1..%d)\n",
+        ERROR("Group index for %s of key %s is out of range (1..%d)\n",
                name, longText(key->name, XkbMessage), XkbNumKbdGroups + 1);
-        ACTION2("Ignoring %s for group %d\n", name, tmp.uval);
+        ACTION("Ignoring %s for group %d\n", name, tmp.uval);
         return False;
     }
     *ndx_rtrn = tmp.uval - 1;
@@ -928,7 +929,7 @@ GetGroupIndex(KeyInfo * key,
 static Bool
 AddSymbolsToKey(KeyInfo * key,
                 XkbDescPtr xkb,
-                char *field,
+                const char *field,
                 ExprDef * arrayNdx, ExprDef * value, SymbolsInfo * info)
 {
     unsigned ndx, nSyms;
@@ -943,15 +944,15 @@ AddSymbolsToKey(KeyInfo * key,
     }
     if (value->op != ExprKeysymList)
     {
-        ERROR1("Expected a list of symbols, found %s\n",
+        ERROR("Expected a list of symbols, found %s\n",
                exprOpText(value->op));
-        ACTION2("Ignoring symbols for group %d of %s\n", ndx,
+        ACTION("Ignoring symbols for group %d of %s\n", ndx,
                 longText(key->name, XkbMessage));
         return False;
     }
     if (key->syms[ndx] != NULL)
     {
-        WSGO2("Symbols for key %s, group %d already defined\n",
+        WSGO("Symbols for key %s, group %d already defined\n",
               longText(key->name, XkbMessage), ndx);
         return False;
     }
@@ -959,7 +960,7 @@ AddSymbolsToKey(KeyInfo * key,
     if (((key->numLevels[ndx] < nSyms) || (key->syms[ndx] == NULL)) &&
         (!ResizeKeyGroup(key, ndx, nSyms, False)))
     {
-        WSGO2("Could not resize group %d of key %s\n", ndx,
+        WSGO("Could not resize group %d of key %s\n", ndx,
               longText(key->name, XkbMessage));
         ACTION("Symbols lost\n");
         return False;
@@ -967,7 +968,7 @@ AddSymbolsToKey(KeyInfo * key,
     key->symsDefined |= (1 << ndx);
     for (i = 0; i < nSyms; i++) {
         if (!LookupKeysym(value->value.list.syms[i], &key->syms[ndx][i])) {
-            WSGO1("Could not resolve keysym %s\n", value->value.list.syms[i]);
+            WARN("Could not resolve keysym %s\n", value->value.list.syms[i]);
             key->syms[ndx][i] = NoSymbol;
         }
     }
@@ -982,7 +983,7 @@ AddSymbolsToKey(KeyInfo * key,
 static Bool
 AddActionsToKey(KeyInfo * key,
                 XkbDescPtr xkb,
-                char *field,
+                const char *field,
                 ExprDef * arrayNdx, ExprDef * value, SymbolsInfo * info)
 {
     register int i;
@@ -1000,14 +1001,14 @@ AddActionsToKey(KeyInfo * key,
     }
     if (value->op != ExprActionList)
     {
-        WSGO1("Bad expression type (%d) for action list value\n", value->op);
-        ACTION2("Ignoring actions for group %d of %s\n", ndx,
+        WSGO("Bad expression type (%d) for action list value\n", value->op);
+        ACTION("Ignoring actions for group %d of %s\n", ndx,
                 longText(key->name, XkbMessage));
         return False;
     }
     if (key->acts[ndx] != NULL)
     {
-        WSGO2("Actions for key %s, group %d already defined\n",
+        WSGO("Actions for key %s, group %d already defined\n",
               longText(key->name, XkbMessage), ndx);
         return False;
     }
@@ -1023,7 +1024,7 @@ AddActionsToKey(KeyInfo * key,
     if (((key->numLevels[ndx] < nActs) || (key->acts[ndx] == NULL)) &&
         (!ResizeKeyGroup(key, ndx, nActs, True)))
     {
-        WSGO2("Could not resize group %d of key %s\n", ndx,
+        WSGO("Could not resize group %d of key %s\n", ndx,
               longText(key->name, XkbMessage));
         ACTION("Actions lost\n");
         return False;
@@ -1036,9 +1037,9 @@ AddActionsToKey(KeyInfo * key,
     {
         if (!HandleActionDef(act, xkb, toAct, MergeOverride, info->action))
         {
-            ERROR1("Illegal action definition for %s\n",
+            ERROR("Illegal action definition for %s\n",
                    longText(key->name, XkbMessage));
-            ACTION2("Action for group %d/level %d ignored\n", ndx + 1, i + 1);
+            ACTION("Action for group %d/level %d ignored\n", ndx + 1, i + 1);
         }
         act = (ExprDef *) act->common.next;
     }
@@ -1065,9 +1066,9 @@ SetAllowNone(KeyInfo * key, ExprDef * arrayNdx, ExprDef * value)
         }
         if ((tmp.uval < 1) || (tmp.uval > XkbMaxRadioGroups))
         {
-            ERROR1("Illegal radio group specified (must be 1..%d)\n",
+            ERROR("Illegal radio group specified (must be 1..%d)\n",
                    XkbMaxRadioGroups + 1);
-            ACTION1("Value of \"allow none\" for group %d ignored\n",
+            ACTION("Value of \"allow none\" for group %d ignored\n",
                     tmp.uval);
             return False;
         }
@@ -1075,7 +1076,7 @@ SetAllowNone(KeyInfo * key, ExprDef * arrayNdx, ExprDef * value)
     }
     if (!ExprResolveBoolean(value, &tmp, NULL, NULL))
     {
-        ERROR1("Illegal \"allow none\" value for %s\n",
+        ERROR("Illegal \"allow none\" value for %s\n",
                longText(key->name, XkbMessage));
         ACTION("Non-boolean value ignored\n");
         return False;
@@ -1118,7 +1119,7 @@ static LookupEntry rgEntries[] = {
 static Bool
 SetSymbolsField(KeyInfo * key,
                 XkbDescPtr xkb,
-                char *field,
+                const char *field,
                 ExprDef * arrayNdx, ExprDef * value, SymbolsInfo * info)
 {
     Bool ok = True;
@@ -1141,17 +1142,17 @@ SetSymbolsField(KeyInfo * key,
         else if (!ExprResolveInteger(arrayNdx, &ndx, SimpleLookup,
                                      (XPointer) groupNames))
         {
-            ERROR1("Illegal group index for type of key %s\n",
+            ERROR("Illegal group index for type of key %s\n",
                    longText(key->name, XkbMessage));
             ACTION("Definition with non-integer array index ignored\n");
             return False;
         }
         else if ((ndx.uval < 1) || (ndx.uval > XkbNumKbdGroups))
         {
-            ERROR2
+            ERROR
                 ("Group index for type of key %s is out of range (1..%d)\n",
                  longText(key->name, XkbMessage), XkbNumKbdGroups + 1);
-            ACTION1("Ignoring type for group %d\n", ndx.uval);
+            ACTION("Ignoring type for group %d\n", ndx.uval);
             return False;
         }
         else
@@ -1176,9 +1177,9 @@ SetSymbolsField(KeyInfo * key,
         }
         else
         {
-            ERROR1("Expected a virtual modifier mask, found %s\n",
+            ERROR("Expected a virtual modifier mask, found %s\n",
                    exprOpText(value->op));
-            ACTION1("Ignoring virtual modifiers definition for key %s\n",
+            ACTION("Ignoring virtual modifiers definition for key %s\n",
                     longText(key->name, XkbMessage));
         }
     }
@@ -1201,7 +1202,7 @@ SetSymbolsField(KeyInfo * key,
                                 (XPointer) rgEntries);
         if (!ok)
         {
-            ERROR1("Illegal radio group specification for %s\n",
+            ERROR("Illegal radio group specification for %s\n",
                    longText(key->name, XkbMessage));
             ACTION("Non-integer radio group ignored\n");
             return False;
@@ -1214,10 +1215,10 @@ SetSymbolsField(KeyInfo * key,
         }
         if ((tmp.uval < 1) || (tmp.uval > XkbMaxRadioGroups))
         {
-            ERROR1
+            ERROR
                 ("Radio group specification for %s out of range (1..32)\n",
                  longText(key->name, XkbMessage));
-            ACTION1("Illegal radio group %d ignored\n", tmp.uval);
+            ACTION("Illegal radio group %d ignored\n", tmp.uval);
             return False;
         }
         key->behavior.type =
@@ -1235,7 +1236,7 @@ SetSymbolsField(KeyInfo * key,
              uStrCasePrefix("permanentoverlay", field))
     {
         Bool permanent = False;
-        char *which;
+        const char *which;
         int overlayNdx;
         if (uStrCasePrefix("permanent", field))
         {
@@ -1250,7 +1251,7 @@ SetSymbolsField(KeyInfo * key,
         {
             if (((overlayNdx < 1) || (overlayNdx > 2)) && (warningLevel > 0))
             {
-                ERROR2("Illegal overlay %d specified for %s\n",
+                ERROR("Illegal overlay %d specified for %s\n",
                        overlayNdx, longText(key->name, XkbMessage));
                 ACTION("Ignored\n");
                 return False;
@@ -1260,7 +1261,7 @@ SetSymbolsField(KeyInfo * key,
             overlayNdx = 1;
         else if (warningLevel > 0)
         {
-            ERROR2("Illegal overlay \"%s\" specified for %s\n",
+            ERROR("Illegal overlay \"%s\" specified for %s\n",
                    which, longText(key->name, XkbMessage));
             ACTION("Ignored\n");
             return False;
@@ -1268,7 +1269,7 @@ SetSymbolsField(KeyInfo * key,
         ok = ExprResolveKeyName(value, &tmp, NULL, NULL);
         if (!ok)
         {
-            ERROR1("Illegal overlay key specification for %s\n",
+            ERROR("Illegal overlay key specification for %s\n",
                    longText(key->name, XkbMessage));
             ACTION("Overlay key must be specified by name\n");
             return False;
@@ -1291,7 +1292,7 @@ SetSymbolsField(KeyInfo * key,
         ok = ExprResolveEnum(value, &tmp, repeatEntries);
         if (!ok)
         {
-            ERROR1("Illegal repeat setting for %s\n",
+            ERROR("Illegal repeat setting for %s\n",
                    longText(key->name, XkbMessage));
             ACTION("Non-boolean repeat setting ignored\n");
             return False;
@@ -1305,7 +1306,7 @@ SetSymbolsField(KeyInfo * key,
         ok = ExprResolveBoolean(value, &tmp, NULL, NULL);
         if (!ok)
         {
-            ERROR1("Illegal groupsWrap setting for %s\n",
+            ERROR("Illegal groupsWrap setting for %s\n",
                    longText(key->name, XkbMessage));
             ACTION("Non-boolean value ignored\n");
             return False;
@@ -1322,7 +1323,7 @@ SetSymbolsField(KeyInfo * key,
         ok = ExprResolveBoolean(value, &tmp, NULL, NULL);
         if (!ok)
         {
-            ERROR1("Illegal groupsClamp setting for %s\n",
+            ERROR("Illegal groupsClamp setting for %s\n",
                    longText(key->name, XkbMessage));
             ACTION("Non-boolean value ignored\n");
             return False;
@@ -1339,16 +1340,16 @@ SetSymbolsField(KeyInfo * key,
         if (!ExprResolveInteger
             (value, &tmp, SimpleLookup, (XPointer) groupNames))
         {
-            ERROR1("Illegal group index for redirect of key %s\n",
+            ERROR("Illegal group index for redirect of key %s\n",
                    longText(key->name, XkbMessage));
             ACTION("Definition with non-integer group ignored\n");
             return False;
         }
         if ((tmp.uval < 1) || (tmp.uval > XkbNumKbdGroups))
         {
-            ERROR2("Out-of-range (1..%d) group for redirect of key %s\n",
+            ERROR("Out-of-range (1..%d) group for redirect of key %s\n",
                    XkbNumKbdGroups, longText(key->name, XkbMessage));
-            ERROR1("Ignoring illegal group %d\n", tmp.uval);
+            ERROR("Ignoring illegal group %d\n", tmp.uval);
             return False;
         }
         key->groupInfo =
@@ -1357,7 +1358,7 @@ SetSymbolsField(KeyInfo * key,
     }
     else
     {
-        ERROR1("Unknown field %s in a symbol interpretation\n", field);
+        ERROR("Unknown field %s in a symbol interpretation\n", field);
         ACTION("Definition ignored\n");
         ok = False;
     }
@@ -1384,16 +1385,16 @@ SetGroupName(SymbolsInfo * info, ExprDef * arrayNdx, ExprDef * value)
     }
     if ((tmp.uval < 1) || (tmp.uval > XkbNumKbdGroups))
     {
-        ERROR1
+        ERROR
             ("Attempt to specify name for illegal group (must be 1..%d)\n",
              XkbNumKbdGroups + 1);
-        ACTION1("Name for group %d ignored\n", tmp.uval);
+        ACTION("Name for group %d ignored\n", tmp.uval);
         return False;
     }
     if (!ExprResolveString(value, &name, NULL, NULL))
     {
         ERROR("Group name must be a string\n");
-        ACTION1("Illegal name for group %d ignored\n", tmp.uval);
+        ACTION("Illegal name for group %d ignored\n", tmp.uval);
         return False;
     }
     info->groupNames[tmp.uval - 1 + info->explicit_group] =
@@ -1466,10 +1467,10 @@ HandleSymbolsVar(VarDef * stmt, XkbDescPtr xkb, SymbolsInfo * info)
         }
         if ((tmp.uval < 1) || (tmp.uval > XkbNumKbdGroups))
         {
-            ERROR1
+            ERROR
                 ("Out-of-range (1..%d) group for global groupsRedirect\n",
                  XkbNumKbdGroups);
-            ACTION1("Ignoring illegal group %d\n", tmp.uval);
+            ACTION("Ignoring illegal group %d\n", tmp.uval);
             return False;
         }
         info->groupInfo = XkbSetGroupInfo(0, XkbRedirectIntoRange, tmp.uval);
@@ -1532,8 +1533,8 @@ SetExplicitGroup(SymbolsInfo * info, KeyInfo * key)
     if ((key->typesDefined | key->symsDefined | key->actsDefined) & ~1)
     {
         int i;
-        WARN1("For the map %s an explicit group specified\n", info->name);
-        WARN1("but key %s has more than one group defined\n",
+        WARN("For the map %s an explicit group specified\n", info->name);
+        WARN("but key %s has more than one group defined\n",
               longText(key->name, XkbMessage));
         ACTION("All groups except first one will be ignored\n");
         for (i = 1; i < XkbNumKbdGroups; i++)
@@ -1603,7 +1604,7 @@ HandleModMapDef(ModMapDef * def,
     if (!LookupModIndex(NULL, None, def->modifier, TypeInt, &rtrn))
     {
         ERROR("Illegal modifier map definition\n");
-        ACTION1("Ignoring map for non-modifier \"%s\"\n",
+        ACTION("Ignoring map for non-modifier \"%s\"\n",
                 XkbAtomText(NULL, def->modifier, XkbMessage));
         return False;
     }
@@ -1624,7 +1625,7 @@ HandleModMapDef(ModMapDef * def,
         else
         {
             ERROR("Modmap entries may contain only key names or keysyms\n");
-            ACTION1("Illegal definition for %s modifier ignored\n",
+            ACTION("Illegal definition for %s modifier ignored\n",
                     XkbModIndexText(tmp.modifier, XkbMessage));
             continue;
         }
@@ -1678,7 +1679,7 @@ HandleSymbolsFile(XkbFile * file,
                 info->errorCount++;
             break;
         default:
-            WSGO1("Unexpected statement type %d in HandleSymbolsFile\n",
+            WSGO("Unexpected statement type %d in HandleSymbolsFile\n",
                   stmt->stmtType);
             break;
         }
@@ -1688,7 +1689,7 @@ HandleSymbolsFile(XkbFile * file,
 #ifdef NOISY
             ERROR("Too many errors\n");
 #endif
-            ACTION1("Abandoning symbols file \"%s\"\n", file->topName);
+            ACTION("Abandoning symbols file \"%s\"\n", file->topName);
             break;
         }
     }
@@ -1897,7 +1898,7 @@ PrepareKeyDef(KeyInfo * key)
         }
     }
     /* If all groups are completely identical remove them all */
-    /* exept the first one. */
+    /* except the first one. */
     identical = True;
     for (i = lastGroup; i > 0; i--)
     {
@@ -1970,7 +1971,7 @@ CopySymbolsDef(XkbFileInfo * result, KeyInfo * key, int start_from)
     {
         if ((start_from == 0) && (warningLevel >= 5))
         {
-            WARN2("Key %s not found in %s keycodes\n",
+            WARN("Key %s not found in %s keycodes\n",
                   longText(key->name, XkbMessage),
                   XkbAtomText(NULL, xkb->names->keycodes, XkbMessage));
             ACTION("Symbols ignored\n");
@@ -2001,9 +2002,9 @@ CopySymbolsDef(XkbFileInfo * result, KeyInfo * key, int start_from)
             {
                 if (warningLevel >= 5)
                 {
-                    WARN1("No automatic type for %d symbols\n",
+                    WARN("No automatic type for %d symbols\n",
                           (unsigned int) key->numLevels[i]);
-                    ACTION3("Using %s for the %s key (keycode %d)\n",
+                    ACTION("Using %s for the %s key (keycode %d)\n",
                             XkbAtomText(NULL, key->types[i],
                                         XkbMessage),
                             longText(key->name, XkbMessage), kc);
@@ -2019,9 +2020,9 @@ CopySymbolsDef(XkbFileInfo * result, KeyInfo * key, int start_from)
         {
             if (warningLevel >= 3)
             {
-                WARN1("Type \"%s\" is not defined\n",
+                WARN("Type \"%s\" is not defined\n",
                       XkbAtomText(NULL, key->types[i], XkbMessage));
-                ACTION2("Using TWO_LEVEL for the %s key (keycode %d)\n",
+                ACTION("Using TWO_LEVEL for the %s key (keycode %d)\n",
                         longText(key->name, XkbMessage), kc);
             }
             types[i] = XkbTwoLevelIndex;
@@ -2030,9 +2031,9 @@ CopySymbolsDef(XkbFileInfo * result, KeyInfo * key, int start_from)
         type = &xkb->map->types[types[i]];
         if (type->num_levels < key->numLevels[i])
         {
-            if (warningLevel > 0)
+            if (warningLevel > 5)
             {
-                WARN4
+                WARN
                     ("Type \"%s\" has %d levels, but %s has %d symbols\n",
                      XkbAtomText(NULL, type->name, XkbMessage),
                      (unsigned int) type->num_levels,
@@ -2054,7 +2055,7 @@ CopySymbolsDef(XkbFileInfo * result, KeyInfo * key, int start_from)
     outSyms = XkbResizeKeySyms(xkb, kc, i);
     if (outSyms == NULL)
     {
-        WSGO2("Could not enlarge symbols for %s (keycode %d)\n",
+        WSGO("Could not enlarge symbols for %s (keycode %d)\n",
               longText(key->name, XkbMessage), kc);
         return False;
     }
@@ -2063,7 +2064,7 @@ CopySymbolsDef(XkbFileInfo * result, KeyInfo * key, int start_from)
         outActs = XkbResizeKeyActions(xkb, kc, i);
         if (outActs == NULL)
         {
-            WSGO2("Could not enlarge actions for %s (key %d)\n",
+            WSGO("Could not enlarge actions for %s (key %d)\n",
                   longText(key->name, XkbMessage), kc);
             return False;
         }
@@ -2083,7 +2084,7 @@ CopySymbolsDef(XkbFileInfo * result, KeyInfo * key, int start_from)
         /* assign kt_index[i] to the index of the type in map->types.
          * kt_index[i] may have been set by a previous run (if we have two
          * layouts specified). Let's not overwrite it with the ONE_LEVEL
-         * default group if we dont even have keys for this group anyway.
+         * default group if we don't even have keys for this group anyway.
          *
          * FIXME: There should be a better fix for this.
          */
@@ -2123,10 +2124,10 @@ CopySymbolsDef(XkbFileInfo * result, KeyInfo * key, int start_from)
         {
             if (warningLevel >= 1)
             {
-                WARN2("Key %s not found in %s keycodes\n",
+                WARN("Key %s not found in %s keycodes\n",
                       longText(key->nameForOverlayKey, XkbMessage),
                       XkbAtomText(NULL, xkb->names->keycodes, XkbMessage));
-                ACTION1("Not treating %s as an overlay key \n",
+                ACTION("Not treating %s as an overlay key \n",
                         longText(key->name, XkbMessage));
             }
             break;
@@ -2170,10 +2171,10 @@ CopyModMapDef(XkbFileInfo * result, ModMapEntry * entry)
     {
         if (warningLevel >= 5)
         {
-            WARN2("Key %s not found in %s keycodes\n",
+            WARN("Key %s not found in %s keycodes\n",
                   longText(entry->u.keyName, XkbMessage),
                   XkbAtomText(NULL, xkb->names->keycodes, XkbMessage));
-            ACTION1("Modifier map entry for %s not updated\n",
+            ACTION("Modifier map entry for %s not updated\n",
                     XkbModIndexText(entry->modifier, XkbMessage));
         }
         return False;
@@ -2183,10 +2184,10 @@ CopyModMapDef(XkbFileInfo * result, ModMapEntry * entry)
     {
         if (warningLevel > 5)
         {
-            WARN2("Key \"%s\" not found in %s symbol map\n",
+            WARN("Key \"%s\" not found in %s symbol map\n",
                   XkbKeysymText(entry->u.keySym, XkbMessage),
                   XkbAtomText(NULL, xkb->names->symbols, XkbMessage));
-            ACTION1("Modifier map entry for %s not updated\n",
+            ACTION("Modifier map entry for %s not updated\n",
                     XkbModIndexText(entry->modifier, XkbMessage));
         }
         return False;
@@ -2280,8 +2281,7 @@ CompileSymbols(XkbFile * file, XkbFileInfo * result, unsigned merge)
                     char buf[5];
                     memcpy(buf, xkb->names->keys[i].name, 4);
                     buf[4] = '\0';
-                    WARN2
-                        ("No symbols defined for <%s> (keycode %d)\n",
+                    INFO("No symbols defined for <%s> (keycode %d)\n",
                          buf, i);
                 }
             }
